a.shopItemDetailImage {
    float: left;
    padding-right: 10px;
    padding-bottom: 10px;
}

table.shopItemListingTable {
    width: 100%;
}

td.shopItemImage {
    width: 128px;
    min-width: 128px;
    text-align: center;
    padding-right: 20px;
}


.variation-details {
    font-size: 90%;
    margin-left: 2em;
}

.shopItem {
    border: 1px solid #E5E5E5;
    margin-bottom: 1em;
    padding: 5px;
    padding-bottom: 0px;
    background-color: #F8F8F8;
}

.shopItem table.itemDataTable {
    margin-bottom: 0;
}

.shopItem table td, .shopItemTable td, .cartListing td {
    padding-left: 4px;
    padding-right: 4px;
}

.shopItem table th, .shopItemTable th, .cartListing th {
    padding-left: 4px;
    padding-right: 4px;
}

#content .shopItem dl.portalMessage {
    font-size: 90%;
    margin-top: 0;
    margin-bottom: 5px;
}

.addToCartLink {
    background:url("++resource++ftw-shop-resources/addcart_p.gif") no-repeat scroll left top transparent;
    float:right;
    padding-left:25px;
    text-decoration:none;
}

#variations label { width: 10em; float: left; }
#variations label.error { float: none; color: red; padding-left: .5em; vertical-align: top; font-size: 70%; position: absolute;}
#variations p { clear: both; }
#variations .submit { margin-left: 12em; }
#variations em { font-weight: bold; padding-right: 1em; vertical-align: top; }

#variations table.listing th,
#variations table.listing td {
    padding: 0.5em 0.1em;
    vertical-align: top;
}

#variations {

    padding: 0.5em;
    border: 1px solid #ddd;
}

.ui-sortable-helper {
	border: 1px solid grey;

}

.add-variant,
.del-variant,
.drag-variant {
    cursor: pointer;
}
.clearer {
	clear: both;}

/* root element for tabs  */
ul.wizard-steps {
	margin:0 !important;
	padding:0;
	height:30px;
	border-bottom:1px solid #666;
	font-site: 90%;
}

/* single tab */
ul.wizard-steps li {
	float:left;
	padding:0;
	margin:0;
	list-style-type:none;

	float:left;
	font-size:13px;
	display:block;
	padding:5px 10px;
	text-decoration:none;
	border:1px solid #666;
	border-bottom:0px;
	height:18px;
	margin-right:2px;
	position:relative;
	top:1px;
	outline:0;
	-moz-border-radius:4px 4px 0 0;
}

/* link inside the tab. uses a background image */
ul.wizard-steps a {
/*	background-color:#efefef;*/
	color:#777;
}

ul.wizard-steps a:hover {
	background-color:#F7F7F7;
	color:#333;
}

/* selected tab */
ul.wizard-steps li.selected {
	background-color:#eee;
	border-bottom:1px solid #eee;
	color:#000;
	cursor:default;
}

/* tab pane */
.tab-pane {
	border:1px solid #666;
	border-width:0 1px 1px 1px;
	min-height:150px;
	padding:15px 20px;
	background-color: #eee;
}

.tab-pane .fieldRequired {
	color: #eee !important;
}

table.padded-table td, table.padded-table th {
	padding-left: 10px;
	padding-right: 10px
}

.greyed-out {
	color: grey !important;
}


table.categoryListing {
    width: 100%;
}

table.categoryListing th,
table.categoryListing td {
    padding: 0.2em;
}

table.categoryListing .toggle,
table.categoryListing .check {
    width: 20px;
}
table.categoryListing .rank {
    width: 110px;
}
table.categoryListing .topItem {
    font-weight: bold;
}

table.categoryListing tr.odd.active td {
    background-color: #ffeecc !important;
}
table.categoryListing tr.even.active td {
    background-color: #ffeeaa !important;
}
table.categoryListing .toggleCategory a {
    text-decoration: none !important;
}

/* =========== New Variations =============== */
.varbox {
    border: 1px solid #ccc;
    background-color: #ddd;
    margin-bottom: 1em;
}

.centered {
    text-align: center;
}

.width100 {
    width: 100%;
}

.panel-body {
    border: 1px solid #ccc;
    background-color: #eee;
    margin-bottom: 1em;
    padding: 1em;
}

.panel-header {
    background-color: #ccc;
    margin-bottom: 0 !important;
    padding-left: 1em;
}


form#variations table td {
    font-size: 80%;
}
form#variations table th {
    font-size: 80%;
    text-align: center;
}
form#variations legend {
    font-size: 120%;
}

.attribute-box {
    float: left;
    padding: 1em;
}

/*.status-1 {
    background-color: #ddbbbb;
}

.status-2 {
    background-color: #ffcc88;
}

.status-3 {
    background-color: #88dd88;
}
*/

/* Archetypes widgets with name 'title' are styled large and bold by default, override */

input#shipping_address-widgets-title, input#contact_information-widgets-title {
    font-size: 100% !important;
    font-weight: normal !important;
}

.variation-toplevel-group{
    clear: both;
}

.template-cart_edit td span {
    display: block;
    overflow: auto;
    margin: 1px 0;
}
.template-cart_edit td span input {
    float: right;
}

.template-checkout-wizard .cartListing .dimensions-display {
    white-space: nowrap;
    display: inline-block;
}
